#define _POSIX_C_SOURCE 200112L
#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <mpc.h>
#include <stdlib.h>
#include <sys/types.h>          /* for cputime */
#include <sys/resource.h>       /* for cputime */

#include "cmh.h"

uint64_t microseconds()
{
    struct rusage res[1];
    getrusage(RUSAGE_SELF, res);
    uint64_t r;
    r = (uint64_t) res->ru_utime.tv_sec;
    r *= (uint64_t) 1000000UL;
    r += (uint64_t) res->ru_utime.tv_usec;
    return r;
}

double cputime()
{
    double milli = (microseconds() / (uint64_t) 1000);
    return milli / 1.0e3;
}

void usage(const char * progname) {
    fprintf(stderr, "Usage: %s [function] [precision]\n", progname);
    exit(1);
}

int main(int argc, char * argv[])
{
    int add_time = 0;
    double tt = 0;
    int i;

    if (argc < 2) usage(argv[0]);

    if (strcmp(argv[1], "time") == 0) {
        add_time = 1;
        argv++,argc--;
    }

    if (argc != 3) usage(argv[0]);

    int prec = atoi(argv[2]);

    if (prec == 0) usage(argv[0]);

    mpc_t tau[3];
    mpc_t th2[10];
    mpc_t h[4];
    mpc_t I[4];
    mpc_t jinv[3];
    mpc_t b[3];
    for(i = 0 ; i < 3 ; i++) mpc_init2(tau[i], prec);
    for(i = 0 ; i < 10 ; i++) mpc_init2(th2[i], prec);
    for(i = 0 ; i < 4 ; i++) mpc_init2(h[i], prec);
    for(i = 0 ; i < 4 ; i++) mpc_init2(I[i], prec);
    for(i = 0 ; i < 3 ; i++) mpc_init2(jinv[i], prec);
    for(i = 0 ; i < 3 ; i++) mpc_init2(b[i], prec);

    if (strcmp(argv[1], "10theta2") == 0) {
        /* 10 theta-squares */
        size_t read = 0;
        for(i = 0 ; i < 3 ; i++) {
            mpc_inp_str(tau[i], stdin, &read, 0, MPC_RNDNN);
        }
        tt -= cputime();
        eval_10theta2_naive(th2, tau);
        tt += cputime();
        for(i = 0 ; i < 10 ; i++) {
            mpc_out_str(stdout, 10, 0, th2[i], MPC_RNDNN);
            printf("\n");
        }
    } else if (strcmp(argv[1], "10theta2_newton") == 0) {
        /* 10 theta-squares */
        size_t read = 0;
        for(i = 0 ; i < 3 ; i++) {
            mpc_inp_str(tau[i], stdin, &read, 0, MPC_RNDNN);
        }
        tt -= cputime();
        eval_10theta2_newton(th2, tau, 0, 0);
        tt += cputime();
        for(i = 0 ; i < 10 ; i++) {
            mpc_out_str(stdout, 10, 0, th2[i], MPC_RNDNN);
            printf("\n");
        }
    } else if(strcmp(argv[1], "4theta") == 0) {
        /* 4 theta (not squares) */
        size_t read = 0;
        for(i = 0 ; i < 3 ; i++) {
            mpc_inp_str(tau[i], stdin, &read, 0, MPC_RNDNN);
        }
        tt -= cputime();
        eval_4theta_naive(th2, tau);
        tt += cputime();
        for(i = 0 ; i < 4 ; i++) {
            mpc_out_str(stdout, 10, 0, th2[i], MPC_RNDNN);
            printf("\n");
        }
    } else if(strcmp(argv[1], "3theta2q") == 0) {
        /* 3 theta-squares quotients */
        size_t read = 0;
        for(i = 0 ; i < 3 ; i++) {
            mpc_inp_str(tau[i], stdin, &read, 0, MPC_RNDNN);
        }
        tt -= cputime();
        eval_3theta2q_naive(b, tau);
        tt += cputime();
        for(i = 0 ; i < 3 ; i++) {
            mpc_out_str(stdout, 10, 0, b[i], MPC_RNDNN);
            printf("\n");
        }
    } else if(strcmp(argv[1], "10theta2x_from_3thetaqtauhalf") == 0) {
        size_t read = 0;
        for(i = 0 ; i < 3 ; i++) {
            mpc_inp_str(b[i], stdin, &read, 0, MPC_RNDNN);
        }
        tt -= cputime();
        get_10theta2x_from_3thetaqtauhalf(th2, b);
        tt += cputime();
        for(i = 0 ; i < 10 ; i++) {
            mpc_out_str(stdout, 10, 0, th2[i], MPC_RNDNN);
            printf("\n");
        }
    } else if(strcmp(argv[1], "h4h10h12h16") == 0) {
        size_t read = 0;
        for(i = 0 ; i < 3 ; i++) {
            mpc_inp_str(tau[i], stdin, &read, 0, MPC_RNDNN);
        }
        tt -= cputime();
        eval_10theta2_naive(th2, tau);
        tt += cputime();
        h4h10h12h16_from_th2(h, th2);
        for(i = 0 ; i < 4 ; i++) {
            mpc_out_str(stdout, 10, 0, h[i], MPC_RNDNN);
            printf("\n");
        }
    } else if(strcmp(argv[1], "I2I4I6I10") == 0) {
        size_t read = 0;
        for(i = 0 ; i < 3 ; i++) {
            mpc_inp_str(tau[i], stdin, &read, 0, MPC_RNDNN);
        }
        tt -= cputime();
        eval_10theta2_naive(th2, tau);
        tt += cputime();
        h4h10h12h16_from_th2(h, th2);
        I2I4I6I10_from_h4h10h12h16(I, h);
        for(i = 0 ; i < 4 ; i++) {
            mpc_out_str(stdout, 10, 0, I[i], MPC_RNDNN);
            printf("\n");
        }
    } else if(strcmp(argv[1], "streng") == 0) {
        size_t read = 0;
        for(i = 0 ; i < 3 ; i++) {
            mpc_inp_str(tau[i], stdin, &read, 0, MPC_RNDNN);
        }
        tt -= cputime();
        eval_10theta2_naive(th2, tau);
        tt += cputime();
        h4h10h12h16_from_th2(h, th2);
        I2I4I6I10_from_h4h10h12h16(I, h);
        streng_i1i2i3_from_I2I4I6I10(jinv, I);
        for(i = 0 ; i < 3 ; i++) {
            mpc_out_str(stdout, 10, 0, jinv[i], MPC_RNDNN);
            printf("\n");
        }
    } else if(strcmp(argv[1], "b2") == 0) {
        size_t read = 0;
        for(i = 0 ; i < 4 ; i++) {
            mpc_inp_str(h[i], stdin, &read, 0, MPC_RNDNN);
        }
        tt -= cputime();
        BorchardtMean4(b[0], h);
        tt += cputime();
        mpc_out_str(stdout, 10, 0, b[0], MPC_RNDNN);
        printf("\n");
    } else {
        usage(argv[0]);
    }

    if (add_time) {
        printf("%1.7f\n", tt);
    }
    for(i = 0 ; i < 3 ; i++) mpc_clear(tau[i]);
    for(i = 0 ; i < 10 ; i++) mpc_clear(th2[i]);
    for(i = 0 ; i < 4 ; i++) mpc_clear(h[i]);
    for(i = 0 ; i < 4 ; i++) mpc_clear(I[i]);
    for(i = 0 ; i < 3 ; i++) mpc_clear(jinv[i]);
    for(i = 0 ; i < 3 ; i++) mpc_clear(b[i]);

    mpfr_free_cache();
}
