/* parsepari.c -- wrapper for integer I/O in pari/gp
 *
 * Copyright (C) 2012 INRIA
 *
 * This file is part of CMH.
 *
 * CMH is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3 of the License, or (at your
 * option) any later version.
 *
 * CMH is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/ .
 */

#include <stdio.h>
#include "parsepari.h"


FILE* file;
mpz_t z;

/*****************************************************************************/

static GEN mpz_get_Z (mpz_t z)
   /* returns the GEN of type t_INT corresponding to z */

{
   const long l = z->_mp_size;
   const long lz = labs (l) + 2;
   int i;
   GEN x = cgeti (lz);

   x [1] = evalsigne ((l > 0 ? 1 : (l < 0 ? -1 : 0))) | evallgefint (lz);
   for (i = 0; i < labs (l); i++)
      *int_W (x, i) = (z->_mp_d) [i];

   return x;
}

/*****************************************************************************/

void parifopen (const char * filename)
{
   /* opens the file "filename" for reading */
   file = fopen (filename, "r");
   mpz_init (z);
}

/*****************************************************************************/

void parifclose (void)
{
   /* closes the file */
   fclose (file);
   mpz_clear (z);
}

/*****************************************************************************/

GEN parifreadint ()
{
   /* reads the next integer from the file */
   mpz_inp_str (z, file, 10);
   return mpz_get_Z (z);
}

/*****************************************************************************/

GEN parifreadmod (GEN p)
{
   /* reads the next integer from the file and reduces it mod p */
   pari_sp av = avma;
   return gerepileuptoint (av, modii (parifreadint (), p));
}

/*****************************************************************************/

void parifskipint (long n)
{
   /* skips over n integers from the file */
   long i;

   for (i = 0; i < n; i++)
      mpz_inp_str (z, file, 10);
}

/*****************************************************************************/
